=== modified file 'bzrlib/tests/HttpServer.py'
--- bzrlib/tests/HttpServer.py	2006-12-13 12:27:42 +0000
+++ bzrlib/tests/HttpServer.py	2006-12-13 16:29:49 +0000
@@ -105,8 +105,12 @@
             # FIXME: RFC2616 says end is optional and default to file_size
             range_match = self._range_regexp.match(range_str)
             if range_match is not None:
-                ranges.append((int(range_match.group('start')),
-                               int(range_match.group('end'))))
+                start = int(range_match.group('start'))
+                end = int(range_match.group('end'))
+                if start > end:
+                    # Syntactically invalid range
+                    return 0, []
+                ranges.append((start, end))
             else:
                 tail_match = self._tail_regexp.match(range_str)
                 if tail_match is not None:
@@ -179,16 +183,15 @@
         if tail != 0:
             ranges.append((file_size - tail, file_size))
 
-        satisfiable_ranges = True
+        self._satisfiable_ranges = True
         if len(ranges) == 0:
-            satisfiable_ranges = False
+            self._satisfiable_ranges = False
         else:
             def check_range(range_specifier):
                 start, end = range_specifier
-                # RFC2616 14.35, ranges are invalid if start > end
-                # or start > file_size
-                if start > end or start > file_size:
-                    satisfiable_ranges = False
+                # RFC2616 14.35, ranges are invalid if start >= file_size
+                if start >= file_size:
+                    self._satisfiable_ranges = False # Side-effect !
                     return 0, 0
                 # RFC2616 14.35, end values should be truncated
                 # to file_size -1 if they exceed it
@@ -197,7 +200,7 @@
 
             ranges = map(check_range, ranges)
 
-        if not satisfiable_ranges:
+        if not self._satisfiable_ranges:
             # RFC2616 14.16 and 14.35 says that when a server
             # encounters unsatisfiable range specifiers, it
             # SHOULD return a 416.

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-12-01 15:06:29 +0000
+++ bzrlib/tests/test_http.py	2006-12-13 16:29:49 +0000
@@ -621,3 +621,57 @@
     """Tests range requests refusing server for pycurl implementation"""
 
 
+class TestRanges(object):
+    """Test the Range header in GET methods..
+
+    This MUST be used by daughter classes that also inherit from
+    TestCaseWithWebserver.
+
+    We can't inherit directly from TestCaseWithWebserver or the
+    test framework will try to create an instance which cannot
+    run, its implementation being incomplete.
+    """
+
+    def setUp(self):
+        TestCaseWithWebserver.setUp(self)
+        self.build_tree_contents([('a', '0123456789')],)
+        server = self.get_readonly_server()
+        self.transport = self._transport(server.get_url())
+
+    def _file_contents(self, relpath, ranges, tail_amount=0):
+         code, data = self.transport._get(relpath, ranges)
+         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
+         for start, end in ranges:
+             data.seek(start)
+             yield data.read(end - start + 1)
+
+    def _file_tail(self, relpath, tail_amount):
+         code, data = self.transport._get(relpath, [], tail_amount)
+         self.assertTrue(code in (200, 206),'_get returns: %d' % code)
+         data.seek(-tail_amount + 1, 2)
+         return data.read(tail_amount)
+
+    def test_range_header(self):
+        # Valid ranges
+        map(self.assertEqual,['0', '234'],
+            list(self._file_contents('a', [(0,0), (2,4)])),)
+        # Tail
+        self.assertEqual('789', self._file_tail('a', 3))
+        # Syntactically invalid range
+        self.assertRaises(errors.InvalidRange,
+                          self.transport._get, 'a', [(4, 3)])
+        # Semantically invalid range
+        self.assertRaises(errors.InvalidRange,
+                          self.transport._get, 'a', [(42, 128)])
+
+
+class TestRanges_urllib(TestRanges, TestCaseWithWebserver):
+    """Test the Range header in GET methods for urllib implementation"""
+
+    _transport = HttpTransport_urllib
+
+
+class TestRanges_pycurl(TestWithTransport_pycurl,
+                        TestRanges,
+                        TestCaseWithWebserver):
+    """Test the Range header in GET methods for pycurl implementation"""

